/*
 * Copyright (c) 2016 Intel Corporation.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

 #include <zephyr/kernel.h>
 #include <zephyr/drivers/flash.h>
 #include <zephyr/device.h>
 #include <zephyr/devicetree.h>
 #include <zephyr/logging/log.h>
 #include <stdio.h>
 #include <string.h>
 #include <zephyr/drivers/gpio.h>
 
 LOG_MODULE_REGISTER(spi_flash_test, LOG_LEVEL_DBG); // Enable debugging logs
 
 #define SPI_FLASH_TEST_REGION_OFFSET  0x0
 #define SPI_FLASH_SECTOR_SIZE         4096
 
 #define FLASH_NODE DT_NODELABEL(w25q64)
 
 #define LED1_NODE DT_NODELABEL(led1)    // 5V LED
 #define LED0_NODE DT_NODELABEL(led0)    // 3.3V LED

 static const struct gpio_dt_spec led1 = GPIO_DT_SPEC_GET(LED1_NODE, gpios);
 static const struct gpio_dt_spec led0 = GPIO_DT_SPEC_GET(LED0_NODE, gpios);
 
 
 const uint8_t expected[] = { 0x55, 0xAA }; // Changed to uint8_t
 const size_t len = sizeof(expected);
 uint8_t buf[sizeof(expected)]; // Changed to uint8_t
 
 void flash_read_write_test(const struct device *flash_dev)
{
    int rc;
    LOG_INF("\nStarting SPI Flash Read/Write Test...");

    /* Erase the sector before writing */
    LOG_INF("Erasing sector at 0x%X...", SPI_FLASH_TEST_REGION_OFFSET);
    rc = flash_erase(flash_dev, SPI_FLASH_TEST_REGION_OFFSET, SPI_FLASH_SECTOR_SIZE);
    if (rc != 0) {
        LOG_ERR("Flash erase failed! Error: %d", rc);
        return;
    }

    /* Write data to flash */
    LOG_INF("Writing %zu bytes to SPI flash at address 0x%X...", len, SPI_FLASH_TEST_REGION_OFFSET);
    rc = flash_write(flash_dev, SPI_FLASH_TEST_REGION_OFFSET, expected, len);
    if (rc != 0) {
        LOG_ERR("Flash write failed! Error: %d", rc);
        return;
    }

    /* Read data from flash */
    memset(buf, 0, len);
    LOG_INF("Attempting to read from address: 0x%X", SPI_FLASH_TEST_REGION_OFFSET);
    rc = flash_read(flash_dev, SPI_FLASH_TEST_REGION_OFFSET, buf, len);
    if (rc != 0) {
        LOG_ERR("Flash read failed! Error: %d", rc);
        return;
    }

    /* Log the read data */
    LOG_INF("Data read from address 0x%X:", SPI_FLASH_TEST_REGION_OFFSET);
    LOG_HEXDUMP_INF(buf, len, "Read Data:");

    /* Compare data */
    if (memcmp(expected, buf, len) == 0) {
        LOG_INF("Flash Read/Write Test Passed! Data is correct.");
    } else {
        LOG_ERR("Flash Read/Write Test Failed! Data mismatch.");
        LOG_HEXDUMP_INF(expected, len, "Expected Data:");
    }
}

 int main(void)
 {
	 const struct device *flash_dev = DEVICE_DT_GET(FLASH_NODE);
 
	 if (!device_is_ready(flash_dev)) {
		 LOG_ERR("%s: device not ready.", flash_dev->name);
		 return -1;
	 }
 
	 LOG_INF("Initializing GPIOs...");
	 if (!gpio_is_ready_dt(&led0) || !gpio_is_ready_dt(&led1)) {
		 LOG_ERR("GPIOs not ready!");
		 return -1;
	 }
 
	 int ret;
 
	 /* Configure LED GPIOs */
	 ret = gpio_pin_configure_dt(&led0, GPIO_OUTPUT_INACTIVE); // Changed to INACTIVE
	 if (ret < 0) {
		 LOG_ERR("Failed to configure LED0: %d", ret);
		 return ret;
	 }
 
	 ret = gpio_pin_configure_dt(&led1, GPIO_OUTPUT_INACTIVE); // Changed to INACTIVE
	 if (ret < 0) {
		 LOG_ERR("Failed to configure LED1: %d", ret);
		 return ret;
	 }

	 	 LOG_INF("Setting GPIO pins HIGH...");
	 gpio_pin_set_dt(&led0, 1);
	 gpio_pin_set_dt(&led1, 1);
	 	         

 /* Get flash device */
	 LOG_INF("\n%s SPI flash read/write test", flash_dev->name);
	 LOG_INF("==========================");
 
	 /* Perform flash read/write test */
	 flash_read_write_test(flash_dev);
 
	 return 0;
 }
 